==> common-search-1.0-20130926.093238-3-sources.jar: cn/pconline/search/common/log/LogSaveThread.java <==
package cn.pconline.search.common.log;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import cn.pconline.search.common.Configuration;
import cn.pconline.search.common.log.GroupCounter.Item;

/**
 * 日志保存线程，该线程每运行一次将会把昨天的搜索日志进行入库操作
 * 
 * @author zengjie
 * @since 2013-9-23
 * @see
 */
public class LogSaveThread extends Thread
{

    private static Logger logger = Logger.getLogger(LogSaveThread.class);

    private static final int MS_OF_DAY = 24 * 60 * 60 * 1000;

    private static final int WEEK = 1; // 一周的开始,或者本周还没有进行过更新

    private static final int MONTH = 2; // 一个月的开始,或者本月还没有进行过更新

    private static final int YEAR = 3; // 一年的开始,或者本年还没有进行过更新

    private static final int WEEK_MONTH = 4;// 周、月的开始

    private static final int WEEK_YEAR = 5;// 周、年的开始

    private static final int MONTH_YEAR = 6;// 月、年的开始

    private static final int WEEK_MONTH_YEAR = 7;// 周、月、年的开始

    private LogIO logIO;

    private Configuration config;

    private Map<SearchLog, Integer> keyOccurMap;

    private Date runTime;

    /**
     * constructor
     * 
     * @param logIO
     *            日志输入输出对象
     * @param config
     *            相关配置
     * @param runTime
     *            保存日志运行时间点,在每天的指定时间点(通过runTime的hour、minue、second，
     *            millionsecond来指定)
     */
    public LogSaveThread(LogIO logIO, Configuration config)
    {
        super("SearchLog_Save_Thread");
        this.logIO = logIO;
        this.config = config;
    }

    @Override
    public void run()
    {
        // 从配置中获取索引入库的指定运行时间
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String time = config.getConfig("searchLogSaveTime",
                sdf.format(new Date()));
        try
        {
            runTime = sdf.parse(time);
        }
        catch (ParseException e)
        {
        }
        if (runTime == null)
        {
            sdf = new SimpleDateFormat("HH:mm:ss");
            try
            {
                runTime = sdf.parse(time);
            }
            catch (ParseException e)
            {
                runTime = new Date();
            }
        }
        while (true)
        {
            try
            {
                sleepToRun();
            }
            catch (InterruptedException e1)
            {
                return;
            }
            if (Thread.currentThread().isInterrupted())
            {
                return;
            }
            keyOccurMap = new HashMap<SearchLog, Integer>();
            long startTime = System.currentTimeMillis();

            Connection con = null;
            Statement sta = null;
            Statement batchSta = null;

            String adDbUrl = config.getConfig("ad.dburl", null);
            if (adDbUrl == null)
            {
                logger.info("No AD setting for application found.");
                return;
            }
            String user = config.getConfig("ad.user", null);
            String passwd = config.getConfig("ad.passwd", null);
            try
            {
                DriverManager.registerDriver(new oracle.jdbc.OracleDriver());
                con = DriverManager.getConnection(adDbUrl, user, passwd);
                con.setAutoCommit(false);
                sta = con.createStatement();// 查询statement
                batchSta = con.createStatement();// 批处理statment

                getUpdateOrSave(sta, batchSta, Integer.MAX_VALUE, getNowTime());// 读取本地统计文件

                batchSta.executeBatch();
                con.commit();
                long endTime = System.currentTimeMillis();
                logger.info("关键字入库完毕!");
                logger.info("所用时间: " + (endTime - startTime));
            }
            catch (Exception e)
            {
                logger.error("Save search log error", e);
            }
            finally
            {
                try
                {
                    sta.close();
                }
                catch (SQLException e)
                {
                    e.printStackTrace();
                }
                try
                {
                    con.close();
                }
                catch (SQLException e)
                {
                    e.printStackTrace();
                }
            }

        }
    }

    /**
     * 休眠指定时间，确保每天只运行一次
     * 
     * @throws InterruptedException
     */
    private synchronized void sleepToRun() throws InterruptedException
    {
        try
        {
            Date now = new Date();
            while (runTime.getTime() < now.getTime())
            {
                runTime.setTime(runTime.getTime() + MS_OF_DAY);
            }
            long timeToSpleep = (runTime.getTime() - now.getTime());
            logger.debug("Sleep " + timeToSpleep + " ms to run");
            Thread.sleep(timeToSpleep);
            runTime.setTime(runTime.getTime() + MS_OF_DAY);
        }
        catch (InterruptedException e)
        {
            logger.error("sleep interrupted", e);
            throw e;
        }
    }

    protected Statement getUpdateOrSave(Statement sta, Statement batchSta,
            int topNum, int time) throws Exception
    {
        ResultSet rs = null;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        boolean hasUpdate = false;

        List<Item> list = SearchLogHelper.getTop(logIO,
                topNum < 0 ? Integer.MAX_VALUE : topNum, keyOccurMap);

        for (int i = 0, c = list.size(); i < c; ++i)
        {
            if (Thread.currentThread().isInterrupted())
            {
                throw new InterruptedException();
            }
            Item item = list.get(i);
            int count = item.getCount();
            Integer res = keyOccurMap.get(item.getKey());

            String existSql = "select * from SEARCHKEYLOG sl where sl.keyword='"
                    + item.getKey().getKey()
                    + "' and sl.app='"
                    + item.getKey().getApp() + "'";
            rs = sta.executeQuery(existSql);
            if (rs.next())// 已经存在记录
            {
                hasUpdate = sdf.format(rs.getDate("updatetime")).equals(
                        LogIO.getFileKey(LogIO.getYesterday()));
                String updateSql = "update KS_SEARCHKEY_LOG sl "
                        + " SET sl.summary=sl.summary+"
                        + count
                        + ","
                        + // summary: 总数
                        " sl.week_num="
                        + (time == WEEK || time == WEEK_YEAR
                                || time == WEEK_MONTH
                                || time == WEEK_MONTH_YEAR ? ""
                                : "sl.week_num+")
                        + ""
                        + count
                        + ","
                        + // week_num: 本周搜索次数
                        " sl.month_num="
                        + (time == MONTH || time == MONTH_YEAR
                                || time == WEEK_MONTH
                                || time == WEEK_MONTH_YEAR ? ""
                                : "sl.month_num+")
                        + ""
                        + count
                        + ","
                        + // month_num: 本月搜索次数
                        " sl.yesterday_num="
                        + (hasUpdate ? "sl.yesterday_num+" : "")
                        + count
                        + ","
                        + // yesterday_num: 昨天的搜索次数
                        " sl.last_search_num="
                        + res
                        + ","
                        + // last_search_num: 最后一次搜索的结果数
                        " sl.year_num="
                        + (time == YEAR || time == MONTH_YEAR
                                || time == WEEK_YEAR || time == WEEK_MONTH_YEAR ? ""
                                : "sl.year_num+")
                        + count
                        + ","
                        + " sl.updatetime=sysdate-1"
                        + " where sl.keyword='"
                        + item.getKey().getKey()
                        + "' and sl.app='"
                        + item.getKey().getApp() + "'";

                hasUpdate = false;
                batchSta.addBatch(updateSql);
            }
            else
            {
                String insertSql = "insert into KS_SEARCHKEY_LOG(id, keyword, app, summary, createtime, updatetime, week_num, month_num, yesterday_num, year_num, last_search_num)"
                        + " values(SEQ_SERARCHKEY_ID.nextval, '"
                        + item.getKey().getKey()
                        + "','"
                        + item.getKey().getApp()
                        + "',"
                        + count
                        + ",sysdate-1,sysdate-1,"
                        + count
                        + ","
                        + count
                        + ","
                        + count + "," + count + "," + res + ")";
                batchSta.addBatch(insertSql);
            }
        }
        return sta;
    }

    /**
     * 得到当前的前一天日期性质: 一周的开始,或者本周还没有进行过更新, 一个月的开始,或者本月还没有进行过更新, 一年的开始,或者本年还没有进行过更新
     * 
     * @return
     */
    private static int getNowTime()
    {
        Calendar c = Calendar.getInstance();
        c.set(Calendar.DATE, c.get(Calendar.DATE) - 1);// 昨天

        boolean isWeek = c.get(Calendar.DAY_OF_WEEK) - 1 == 1;// 今日星期几(星期一记起为第一天)
        boolean isMonth = c.get(Calendar.DATE) == c
                .getMinimum(Calendar.DAY_OF_MONTH);// 本月的第一天是几号
        boolean isYear = c.get(Calendar.DAY_OF_YEAR) == 1;// 今天是今年的第几天

        if (isWeek && isMonth && isYear) return WEEK_MONTH_YEAR;
        if (isWeek && isMonth) return WEEK_MONTH;
        if (isWeek && isYear) return WEEK_YEAR;
        if (isMonth && isYear) return MONTH_YEAR;
        if (isWeek) return WEEK;
        if (isMonth) return MONTH;
        if (isYear) return YEAR;

        return -1;
    }

}

==> common-search-1.0-20131017.083631-4-sources.jar: cn/pconline/search/common/log/LogSaveThread.java <==
package cn.pconline.search.common.log;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import cn.pconline.search.common.Configuration;
import cn.pconline.search.common.log.GroupCounter.Item;
import cn.pconline.search.common.util.DaliyFixTimeRunThread;

/**
 * 日志保存线程，该线程每运行一次将会把昨天的搜索日志进行入库操作
 * 
 * @author zengjie
 * @since 2013-9-23
 * @see
 */
public class LogSaveThread extends DaliyFixTimeRunThread
{

    private static Logger logger = Logger.getLogger(LogSaveThread.class);

    private static final int WEEK = 1; // 一周的开始,或者本周还没有进行过更新

    private static final int MONTH = 2; // 一个月的开始,或者本月还没有进行过更新

    private static final int YEAR = 3; // 一年的开始,或者本年还没有进行过更新

    private static final int WEEK_MONTH = 4;// 周、月的开始

    private static final int WEEK_YEAR = 5;// 周、年的开始

    private static final int MONTH_YEAR = 6;// 月、年的开始

    private static final int WEEK_MONTH_YEAR = 7;// 周、月、年的开始

    private LogIO logIO;

    private Configuration config;

    private Map<SearchLog, Integer> keyOccurMap;

    /**
     * constructor
     * 
     * @param logIO
     *            日志输入输出对象
     * @param config
     *            相关配置
     * @param runTime
     *            保存日志运行时间点,在每天的指定时间点(通过runTime的hour、minue、second，
     *            millionsecond来指定)
     */
    public LogSaveThread(LogIO logIO, Configuration config)
    {
        super(config.getConfig("searchLogSaveTime"), "SearchLog_Save_Thread");
        this.logIO = logIO;
        this.config = config;
    }

    @Override
    protected void doRun()
    {
        if (Thread.currentThread().isInterrupted())
        {
            return;
        }
        keyOccurMap = new HashMap<SearchLog, Integer>();
        long startTime = System.currentTimeMillis();

        Connection con = null;
        Statement sta = null;
        Statement batchSta = null;

        String adDbUrl = config.getConfig("ad.dburl", null);
        if (adDbUrl == null)
        {
            logger.info("No AD setting for application found.");
            return;
        }
        String user = config.getConfig("ad.user", null);
        String passwd = config.getConfig("ad.passwd", null);
        try
        {
            DriverManager.registerDriver(new oracle.jdbc.OracleDriver());
            con = DriverManager.getConnection(adDbUrl, user, passwd);
            con.setAutoCommit(false);
            sta = con.createStatement();// 查询statement
            batchSta = con.createStatement();// 批处理statment

            getUpdateOrSave(sta, batchSta, Integer.MAX_VALUE, getNowTime());// 读取本地统计文件

            batchSta.executeBatch();
            con.commit();
            long endTime = System.currentTimeMillis();
            logger.info("关键字入库完毕!");
            logger.info("所用时间: " + (endTime - startTime));
        }
        catch (Exception e)
        {
            logger.error("Save search log error", e);
        }
        finally
        {
            try
            {
                sta.close();
            }
            catch (SQLException e)
            {
                e.printStackTrace();
            }
            try
            {
                con.close();
            }
            catch (SQLException e)
            {
                e.printStackTrace();
            }
        }
    }

    protected Statement getUpdateOrSave(Statement sta, Statement batchSta,
            int topNum, int time) throws Exception
    {
        ResultSet rs = null;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        boolean hasUpdate = false;

        List<Item> list = SearchLogHelper.getTop(logIO,
                topNum < 0 ? Integer.MAX_VALUE : topNum, keyOccurMap);

        for (int i = 0, c = list.size(); i < c; ++i)
        {
            if (Thread.currentThread().isInterrupted())
            {
                throw new InterruptedException();
            }
            Item item = list.get(i);
            int count = item.getCount();
            Integer res = keyOccurMap.get(item.getKey());

            String existSql = "select * from SEARCHKEYLOG sl where sl.keyword='"
                    + item.getKey().getKey()
                    + "' and sl.app='"
                    + item.getKey().getApp() + "'";
            rs = sta.executeQuery(existSql);
            if (rs.next())// 已经存在记录
            {
                hasUpdate = sdf.format(rs.getDate("updatetime")).equals(
                        LogIO.getFileKey(LogIO.getYesterday()));
                String updateSql = "update KS_SEARCHKEY_LOG sl "
                        + " SET sl.summary=sl.summary+"
                        + count
                        + ","
                        + // summary: 总数
                        " sl.week_num="
                        + (time == WEEK || time == WEEK_YEAR
                                || time == WEEK_MONTH
                                || time == WEEK_MONTH_YEAR ? ""
                                : "sl.week_num+")
                        + ""
                        + count
                        + ","
                        + // week_num: 本周搜索次数
                        " sl.month_num="
                        + (time == MONTH || time == MONTH_YEAR
                                || time == WEEK_MONTH
                                || time == WEEK_MONTH_YEAR ? ""
                                : "sl.month_num+")
                        + ""
                        + count
                        + ","
                        + // month_num: 本月搜索次数
                        " sl.yesterday_num="
                        + (hasUpdate ? "sl.yesterday_num+" : "")
                        + count
                        + ","
                        + // yesterday_num: 昨天的搜索次数
                        " sl.last_search_num="
                        + res
                        + ","
                        + // last_search_num: 最后一次搜索的结果数
                        " sl.year_num="
                        + (time == YEAR || time == MONTH_YEAR
                                || time == WEEK_YEAR || time == WEEK_MONTH_YEAR ? ""
                                : "sl.year_num+")
                        + count
                        + ","
                        + " sl.updatetime=sysdate-1"
                        + " where sl.keyword='"
                        + item.getKey().getKey()
                        + "' and sl.app='"
                        + item.getKey().getApp() + "'";

                hasUpdate = false;
                batchSta.addBatch(updateSql);
            }
            else
            {
                String insertSql = "insert into KS_SEARCHKEY_LOG(id, keyword, app, summary, createtime, updatetime, week_num, month_num, yesterday_num, year_num, last_search_num)"
                        + " values(SEQ_SERARCHKEY_ID.nextval, '"
                        + item.getKey().getKey()
                        + "','"
                        + item.getKey().getApp()
                        + "',"
                        + count
                        + ",sysdate-1,sysdate-1,"
                        + count
                        + ","
                        + count
                        + ","
                        + count + "," + count + "," + res + ")";
                batchSta.addBatch(insertSql);
            }
        }
        return sta;
    }

    /**
     * 得到当前的前一天日期性质: 一周的开始,或者本周还没有进行过更新, 一个月的开始,或者本月还没有进行过更新, 一年的开始,或者本年还没有进行过更新
     * 
     * @return
     */
    private static int getNowTime()
    {
        Calendar c = Calendar.getInstance();
        c.set(Calendar.DATE, c.get(Calendar.DATE) - 1);// 昨天

        boolean isWeek = c.get(Calendar.DAY_OF_WEEK) - 1 == 1;// 今日星期几(星期一记起为第一天)
        boolean isMonth = c.get(Calendar.DATE) == c
                .getMinimum(Calendar.DAY_OF_MONTH);// 本月的第一天是几号
        boolean isYear = c.get(Calendar.DAY_OF_YEAR) == 1;// 今天是今年的第几天

        if (isWeek && isMonth && isYear) return WEEK_MONTH_YEAR;
        if (isWeek && isMonth) return WEEK_MONTH;
        if (isWeek && isYear) return WEEK_YEAR;
        if (isMonth && isYear) return MONTH_YEAR;
        if (isWeek) return WEEK;
        if (isMonth) return MONTH;
        if (isYear) return YEAR;

        return -1;
    }

}
